/*****************************************************************************
 * ROMAPI_Boot_Core1.c
 *****************************************************************************/

#include <sys/platform.h>
#include <sys/adi_core.h>
#include "adi_initialize.h"
#include "SSL.h"
#include "stdio.h"

#include <cdef2156x_rom.h>
#include <def2156x_rom.h>
#include<sru.h>

#define START_ADDRESS 0x60040000

int32_t ROM_BOOT_HOOK_FUNC1(ADI_ROM_BOOT_CONFIG * pBootConfig, ROM_HOOK_CALL_CAUSE cause);


int main(int argc, char *argv[])
{

//	adi_initComponents();
/*
 * For SSL make sure that stack of the primary application does not coincide with that of secondary booting application.
 * One way to do this is to remap the stack of the primary application to Boot ROM stack space dynamically before calling the ROM API.
 * This is done in the following code snippet. Also cache is disabled for the 8KB region used by the ROM.
 * */

	/*SPI2 memory mapped boot*/
	int i,m;

	*pREG_SHL1C0_CFG2|=0xF00 ;
	*pREG_SHL1C0_RANGE_START4 = 0x200FE000 ;
	*pREG_SHL1C0_RANGE_END4 = 0x200FFFFF ;

	asm("#include <sys/def215xx_core.h>");
	asm("IRPTL = 0;");
	asm("MMASK = BITM_REGF_MMASK_BR8|BITM_REGF_MMASK_BR0|BITM_REGF_MMASK_IRPTEN|BITM_REGF_MMASK_ALUSAT|BITM_REGF_MMASK_TRUNCATE|BITM_REGF_MMASK_PEYEN|BITM_REGF_MMASK_BDCST9|BITM_REGF_MMASK_BDCST1;");
	asm("IMASK = BITM_REGF_IMASK_EMUI|BITM_REGF_IMASK_RSTI|BITM_REGF_IMASK_PARI|BITM_REGF_IMASK_ILOPI|BITM_REGF_IMASK_CB7I|BITM_REGF_IMASK_IICDI|BITM_REGF_IMASK_SOVFI| BITM_REGF_IMASK_ILADI; nop;");
	asm("B7 = 0x200FF400;\
	              I7 = ((0x200FF400 + 0xc00 - 1)-((0x200FF400 + 0xc00 - 1) % 4) );\
	       L7 = (( 0xc00 - ((0x200FF400 + 0xc00 - 1) % 4) )/4);\
	       B6 = B7;\
	       I6 = I7; \
	        L6 = L7;");

	uint32_t BootCmd=0;

	*pREG_SPU0_SECUREP110 = 0x3;
	*pREG_SPU0_SECUREP111 = 0x3;

	BootCmd = (2<<BITP_ROM_BCMD_DEVENUM)|(7<<BITP_ROM_BCMD_DEVICE); /*SPI2 master*/

	/*Toggle LED to verify initial boot is completed*/
	SRU(HIGH,DAI0_PBEN11_I);
	for(m=0;m<5;m++)
	{
		/* Turn LED on */
		for(i=0;i<0x100000;i++);
		SRU(HIGH,DAI0_PB11_I);
		asm("sync;");
		/* Turn LED off */
		for(i=0;i<0x100000;i++);
		SRU(LOW,DAI0_PB11_I);
		asm("sync;");
		for(i=0;i<0x100000;i++);
	}

	/*An hook routine can be used with adi_rom_Boot to customize the boot process*/
	adi_rom_Boot((unsigned int *)START_ADDRESS, BITM_ROM_BFLAG_HOOK,0, &ROM_BOOT_HOOK_FUNC1,BootCmd); /* ROM API is calling the Image stored in Sector 4 of the SPI flash */

	while(1);

}

int32_t ROM_BOOT_HOOK_FUNC1(ADI_ROM_BOOT_CONFIG * pBootConfig, ROM_HOOK_CALL_CAUSE cause)
{
	if (cause == ROM_HOOK_CALL_INIT_COMPLETE)
	{

	}
	if (cause == ROM_HOOK_CALL_CONFIG_COMPLETE)
	{
		/*Customization of boot process can be done here*/
		asm("NOP;");
	}

	return 0;
}

