/*****************************************************************************
 * CGU_DDR_Configurations.c
 *****************************************************************************/

#include <sys/platform.h>
#include "adi_initialize.h"
#include "CGU_DDR_Configurations.h"

#include <cdef2156x_rom.h>
#include <def2156x_rom.h>
#include <stdio.h>
#include <stdlib.h>

#define START_ADDRESS 0x60000000
#define BUFF_SIZE 500

ADI_ROM_OTP_BOOT_CGU_INFO CGU_Config;
ADI_ROM_OTP_DMC_CONFIG DMC_Config;

bool Result;

uint32_t DMC_StartAddress = 0x80000000;
uint8_t  Source[BUFF_SIZE]={0};
uint8_t Destination[BUFF_SIZE]={0};

void WriteReadVerifyDMC(uint32_t COUNT);

int main(int argc, char *argv[])
{

	   adi_initComponents();

	   *pREG_SHL1C0_CFG2|=0xF00 ;
	   *pREG_SHL1C0_RANGE_START4 = 0x200FE000 ;
	   *pREG_SHL1C0_RANGE_END4 = 0x200FFFFF ;

	   asm("#include <sys/def215xx_core.h>");
	   asm("IRPTL = 0;");
	   asm("MMASK = BITM_REGF_MMASK_BR8|BITM_REGF_MMASK_BR0|BITM_REGF_MMASK_IRPTEN|BITM_REGF_MMASK_ALUSAT|BITM_REGF_MMASK_TRUNCATE|BITM_REGF_MMASK_PEYEN|BITM_REGF_MMASK_BDCST9|BITM_REGF_MMASK_BDCST1;");
	   asm("IMASK = BITM_REGF_IMASK_EMUI|BITM_REGF_IMASK_RSTI|BITM_REGF_IMASK_PARI|BITM_REGF_IMASK_ILOPI|BITM_REGF_IMASK_CB7I|BITM_REGF_IMASK_IICDI|BITM_REGF_IMASK_SOVFI| BITM_REGF_IMASK_ILADI; nop;");
	   asm("B7 = 0x200FF400;\
					  I7 = ((0x200FF400 + 0xc00 - 1)-((0x200FF400 + 0xc00 - 1) % 4) );\
			   L7 = (( 0xc00 - ((0x200FF400 + 0xc00 - 1) % 4) )/4);\
			   B6 = B7;\
			   I6 = I7; \
				L6 = L7;");

	   uint32_t BootCmd=0;

	   CGU_Config.ctl_WEN=1;
	   CGU_Config.div_WEN=1;
	   CGU_Config.div_DSEL=3;/*1600/3=533*/
	   CGU_Config.div_CSEL=2;
	   CGU_Config.div_S0SEL=4;
	   CGU_Config.div_SYSSEL=4;
	   CGU_Config.div_S1SEL=2;
	   CGU_Config.div_OSEL=0x10;
	   CGU_Config.ctl_DF=0;
	   CGU_Config.ctl_MSEL=64;/*25*64 = 1600*/
	   CGU_Config.auto_disable=0; /* disable auto alignment*/;
	   CGU_Config.clkoutsel_CLKOUTSEL=0;
	   CGU_Config.clkoutsel_WEN=0;

	   CGU_Config.oscwctl0_WEN=1;
	   CGU_Config.oscwctl0_HODF=0;
	   CGU_Config.oscwctl0_HODEN=0;
	   CGU_Config.oscwctl0_CNGEN=0;
	   CGU_Config.oscwctl0_BOUF=0x1F;
	   CGU_Config.oscwctl0_BOUEN=1;
	   CGU_Config.oscwctl0_FAULTEN=1;
	   CGU_Config.oscwctl0_MONDIS=0;
	   CGU_Config.oscwctl0_FAULTPINDIS=1;

	   /*Perform CGU Initialization using adi_rom_CguInit*/
	   adi_rom_CguInit(&CGU_Config);

	   /*DCLK=533MHz*/

	   DMC_Config.ulDDR_DLLCTLCFG = 0x08f70722; 	 		/*!< Content of DDR DLLCTL and DMC_CFG register     */
	   DMC_Config.ulDDR_EMR2EMR3  = 0x00080000; 	 		/*!< Content of the DDR EMR2 and EMR3 Register      */
	   DMC_Config.ulDDR_CTL 		 = 0x00000a05;  		/*!< Content of the DDR Control                   */
	   DMC_Config.ulDDR_MREMR1	 = 0x0930000c;  			/*!< Content of the DDR MR and EMR1 Register      */
	   DMC_Config.ulDDR_TR0		 = 0x41b14757; 				/*!< Content of the DDR Timing Register      */
	   DMC_Config.ulDDR_TR1 		 = 0x60bb1040;  		/*!< Content of the DDR Timing Register      */
	   DMC_Config.ulDDR_TR2 		 = 0x0034851b; 			/*!< Content of the DDR Timing Register      */
	   DMC_Config.ulDDR_ZQCTL0 	 = 0x00782828; 		 		/*!< Content of ZQCTL0 register */
	   DMC_Config.ulDDR_ZQCTL1 	 = 0x00000000;  			/*!< Content of ZQCTL1 register */
	   DMC_Config.ulDDR_ZQCTL2 	 = 0x70000000; 				/*!< Content of ZQCTL2 register */
	   DMC_Config.ulDDRPHY_CACTL  = 0x00000000;             /*!< Content of DDRPHY_DDR_CA_CTL register */
	   DMC_Config.uBypassDelay_LANE0CTL1 = 0x00000000;      /*!< Content of DDR_LANE0_CTL1[15:10] register */
	   DMC_Config.uBypassDelay_LANE1CTL1 = 0x00000000;      /*!< Content of DDR_LANE1_CTL1[15:10] register */
	   DMC_Config.uBypassDelay_LANE0CTL0 = 0x00000000;      /*!< Content of DDR_LANE0_CTL0[15:10] register */
	   DMC_Config.uBypassDelay_LANE1CTL0 = 0x00000000;      /*!< Content of DDR_LANE1_CTL0[15:10] register */

	   /*Perform DMC Initialization using adi_rom_DmcInit*/
	   Result = adi_rom_DmcPhyCalibration(&DMC_Config,1);
	   Result = adi_rom_DmcInit(&DMC_Config, 1);

	   WriteReadVerifyDMC(BUFF_SIZE);

	   *pREG_SPU0_SECUREP110 = 0x3;
	   *pREG_SPU0_SECUREP111 = 0x3;

	   BootCmd = (2<<BITP_ROM_BCMD_DEVENUM)|(7<<BITP_ROM_BCMD_DEVICE);

	   adi_rom_Boot(START_ADDRESS, 0, 0, 0,BootCmd);

	   while(1);

}


void WriteReadVerifyDMC(unsigned int COUNT)
{
	  int i=0;

	  //Write data to DMC address
	  for(i=0;i<COUNT;i++)
			{
		  	  Source[i]= i+1;
			  *(uint8_t*)(DMC_StartAddress)= Source[i];
			  DMC_StartAddress++;
			}

	  //Re-initialize Start address
	  DMC_StartAddress = 0x80000000;

	  //Read back data from DMC address
	  for(i=0; i<COUNT;i++)
	  {
		  Destination[i]= *(uint8_t*)(DMC_StartAddress);
		  DMC_StartAddress++;
	  }

	  //Compare Write and read data
	  for(i=0;i<COUNT ;i++)
	  {
		  if(Source[i]!= Destination[i])
		  {
			  printf("Test Fail\n");
			  break;
		  }
	  }

	 printf("\nPASS");
}
