/*****************************************************************************
 * ROMAPI_Boot_Core1.c
 *****************************************************************************/

#include <sys/platform.h>
#include <sys/adi_core.h>
#include "adi_initialize.h"
#include "ROMAPI_Boot_Core1.h"
#include "stdio.h"

#include <cdef2156x_rom.h>
#include <def2156x_rom.h>
#include<def21569.h>
#include<cdef21569.h>
#include "sru.h"

#define START_ADDRESS 0x61080000

int32_t ROM_BOOT_HOOK_FUNC1(ADI_ROM_BOOT_CONFIG * pBootConfig, ROM_HOOK_CALL_CAUSE cause) ;

int main(int argc, char *argv[])
{
	int m,i;
/*
 * For SSL make sure that stack of the primary application does not coincide with that of secondary booting application.
 * One way to do this is to remap the stack of the primary application to Boot ROM stack space dynamically before calling the ROM API.
 * This is done in the following code snippet. Also cache is disabled for the 8KB region used by the ROM.
 * */
	*pREG_SHL1C0_CFG2|=0xF00 ;
	*pREG_SHL1C0_RANGE_START4 = 0x200FE000 ;
	*pREG_SHL1C0_RANGE_END4 = 0x200FFFFF ;

	asm("#include <sys/def215xx_core.h>");
	asm("IRPTL = 0;");
	asm("MMASK = BITM_REGF_MMASK_BR8|BITM_REGF_MMASK_BR0|BITM_REGF_MMASK_IRPTEN|BITM_REGF_MMASK_ALUSAT|BITM_REGF_MMASK_TRUNCATE|BITM_REGF_MMASK_PEYEN|BITM_REGF_MMASK_BDCST9|BITM_REGF_MMASK_BDCST1;");
	asm("IMASK = BITM_REGF_IMASK_EMUI|BITM_REGF_IMASK_RSTI|BITM_REGF_IMASK_PARI|BITM_REGF_IMASK_ILOPI|BITM_REGF_IMASK_CB7I|BITM_REGF_IMASK_IICDI|BITM_REGF_IMASK_SOVFI| BITM_REGF_IMASK_ILADI; nop;");
	asm("B7 = 0x200FF400;\
	              I7 = ((0x200FF400 + 0xc00 - 1)-((0x200FF400 + 0xc00 - 1) % 4) );\
	       L7 = (( 0xc00 - ((0x200FF400 + 0xc00 - 1) % 4) )/4);\
	       B6 = B7;\
	       I6 = I7; \
	        L6 = L7;");

	*pREG_SPU0_SECUREP110 = 0x3;
	*pREG_SPU0_SECUREP111 = 0x3;

	/*Toggle LED to verify initial boot is completed*/
	SRU(HIGH,DAI0_PBEN11_I);
	for(m=0;m<5;m++)
	{
		/* Turn LED on */
		for(i=0;i<0x100000;i++);
		SRU(HIGH,DAI0_PB11_I);
		asm("sync;");
		/* Turn LED off */
		for(i=0;i<0x100000;i++);
		SRU(LOW,DAI0_PB11_I);
		asm("sync;");
		for(i=0;i<0x100000;i++);

	}

	/*Use the Hook function to change from 3-byte to 4-byte addressing mode*/
	adi_rom_Boot((unsigned int *)START_ADDRESS, BITM_ROM_BFLAG_HOOK,0, &ROM_BOOT_HOOK_FUNC1,0x10207);

	while(1);

}

int32_t ROM_BOOT_HOOK_FUNC1(ADI_ROM_BOOT_CONFIG * pBootConfig, ROM_HOOK_CALL_CAUSE cause)
{

	if (cause == ROM_HOOK_CALL_CONFIG_COMPLETE)
	{
		ADI_SPI_TypeDef * pSpiRegs = ( ADI_SPI_TypeDef *)pBootConfig->pPeripheralBase;
		ADI_ROM_BOOT_SPI * pSpiStruct = ( ADI_ROM_BOOT_SPI *)pBootConfig->pModeData;

		/* update boot kernel to use memory mapped mode */
		pBootConfig->pDmaBaseRegister = REG_DMA9_DSCPTR_NXT; /* change the DMA we use fromt he SPI peripheral DMA to MDMA for memory mapped mode */

		/*Configure the SPi Memory Mapped registers for 4-byte addressing mode*/
		pSpiRegs-> MMRDH = (((0x13 << BITP_SPI_MMRDH_OPCODE ) 	& BITM_SPI_MMRDH_OPCODE	)	|
						   ((4 		<< BITP_SPI_MMRDH_ADRSIZE) 	& BITM_SPI_MMRDH_ADRSIZE)	|
						   ((0 		<< BITP_SPI_MMRDH_ADRPINS) 	& BITM_SPI_MMRDH_ADRPINS) 	|
						   ((0		<< BITP_SPI_MMRDH_DMYSIZE) 	& BITM_SPI_MMRDH_DMYSIZE) 	|
						   ((0x00 	<< BITP_SPI_MMRDH_MODE	)	& BITM_SPI_MMRDH_MODE	)	|
						   ((1 		<< BITP_SPI_MMRDH_TRIDMY)	& BITM_SPI_MMRDH_TRIDMY	)	|
						   ((1 		<< BITP_SPI_MMRDH_MERGE	)	& BITM_SPI_MMRDH_MERGE	)	|
						   ((0  	<< BITP_SPI_MMRDH_WRAP	)	& BITM_SPI_MMRDH_WRAP	)	|
						   ((0 		<< BITP_SPI_MMRDH_CMDSKIP)	& BITM_SPI_MMRDH_CMDSKIP)	|
						   ((0 		<< BITP_SPI_MMRDH_CMDPINS)	& BITM_SPI_MMRDH_CMDPINS)	);

		/* Update the control value uses to support memory mapped mode */
		pBootConfig->dControlValue = BITM_SPI_CTL_EN | BITM_SPI_CTL_MSTR | (0 << BITP_SPI_CTL_SIZE) | ENUM_SPI_CTL_MM_EN | BITM_SPI_CTL_ASSEL;

		/* Enable both transmit and receive parts of the SPI */
		pSpiRegs->TXCTL |= BITM_SPI_TXCTL_TEN | BITM_SPI_TXCTL_TTI;
		pSpiRegs->RXCTL |= BITM_SPI_RXCTL_REN ;

		/* Enable the SPI in memory mapped mode */
		pSpiRegs->CTL = pBootConfig->dControlValue | pSpiStruct->nRxCtl;

		/* Update our source pointer now to account for the memory mapped location */
		pBootConfig->pSource =  (void *)START_ADDRESS;

		/* change dBootCommand to indicate now using the SPI XIP mode, this is needed for the load function */
		pBootConfig->dBootCommand = ENUM_ROM_BCMD_SPIM_DEVICE_SPIXIP|
									ENUM_ROM_BCMD_SPIM_BCODE_1|
									ENUM_ROM_BCMD_SPIM_DEVENUM_2|
									((1<<BITP_ROM_BCMD_SPIM_NOAUTO)&BITM_ROM_BCMD_SPIM_NOAUTO)|
									((3<<BITP_ROM_BCMD_SPIM_ADDR) & BITM_ROM_BCMD_SPIM_ADDR);
	}
	return 0;
}

