/*****************************************************************************
 * ROM_CustomBoot.c
 *****************************************************************************/

#include <sys/platform.h>
#include "adi_initialize.h"
#include "ROM_CustomBoot.h"

int32_t ROM_BOOT_HOOK_FUNC1(ADI_ROM_BOOT_CONFIG * pBootConfig, ROM_HOOK_CALL_CAUSE cause) ;

ROM_BOOT_RESULT bootInitFunction(ADI_ROM_BOOT_CONFIG * pBootConfig);
ROM_BOOT_RESULT bootConfigFunction(ADI_ROM_BOOT_CONFIG * pBootConfig);
void bootLoadFunction(ADI_ROM_BOOT_CONFIG * pBootConfig);
void bootCleanUpFunction(ADI_ROM_BOOT_CONFIG * pBootConfig);

int main(int argc, char *argv[])
{

	adi_initComponents();
	
	/*
	 * For SSL make sure that stack of the primary application does not coincide with that of secondary booting application.
	 * One way to do this is to remap the stack of the primary application to Boot ROM stack space dynamically before calling the ROM API.
	 * This is done in the following code snippet. Also cache is disabled for the 8KB region used by the ROM.
	 * */

	*pREG_SHL1C0_CFG2|=0xF00 ;
	*pREG_SHL1C0_RANGE_START4 = 0x200FE000 ;
	*pREG_SHL1C0_RANGE_END4 = 0x200FFFFF ;

	asm("#include <sys/def215xx_core.h>");
	asm("IRPTL = 0;");
	asm("MMASK = BITM_REGF_MMASK_BR8|BITM_REGF_MMASK_BR0|BITM_REGF_MMASK_IRPTEN|BITM_REGF_MMASK_ALUSAT|BITM_REGF_MMASK_TRUNCATE|BITM_REGF_MMASK_PEYEN|BITM_REGF_MMASK_BDCST9|BITM_REGF_MMASK_BDCST1;");
	asm("IMASK = BITM_REGF_IMASK_EMUI|BITM_REGF_IMASK_RSTI|BITM_REGF_IMASK_PARI|BITM_REGF_IMASK_ILOPI|BITM_REGF_IMASK_CB7I|BITM_REGF_IMASK_IICDI|BITM_REGF_IMASK_SOVFI| BITM_REGF_IMASK_ILADI; nop;");
	asm("B7 = 0x200FF400;\
	              I7 = ((0x200FF400 + 0xc00 - 1)-((0x200FF400 + 0xc00 - 1) % 4) );\
	       L7 = (( 0xc00 - ((0x200FF400 + 0xc00 - 1) % 4) )/4);\
	       B6 = B7;\
	       I6 = I7; \
	        L6 = L7;");

	uint32_t BootCmd=0;

	*pREG_SPU0_SECUREP110 = 0x3;
	*pREG_SPU0_SECUREP111 = 0x3;

	ConfigSoftSwitches_MacronixFlash();

	BootCmd = (0xF<<BITP_ROM_BCMD_SPIM_DEVICE); /*Custom Boot Support*/

	adi_rom_Boot((unsigned int *)START_ADDRESS, BITM_ROM_BFLAG_HOOK, 0, &ROM_BOOT_HOOK_FUNC1, BootCmd);

	while(1);

}


int32_t ROM_BOOT_HOOK_FUNC1(ADI_ROM_BOOT_CONFIG * pBootConfig, ROM_HOOK_CALL_CAUSE cause)
{
	if (cause == ROM_HOOK_REG_COMPLETE)
	{
		pBootConfig->bootRegistry.pInitFunction = bootInitFunction;
		pBootConfig->bootRegistry.pConfigFunction = bootConfigFunction;
		pBootConfig->bootRegistry.pLoadFunction = bootLoadFunction;
		pBootConfig->bootRegistry.pCleanUpFunction = bootCleanUpFunction;

		asm("NOP;");
	}
	if (cause == ROM_HOOK_CALL_INIT_COMPLETE)
	{
		asm("NOP;");
	}
	if (cause == ROM_HOOK_CALL_CONFIG_COMPLETE)
	{
		pBootConfig->pDestination = pBootConfig->pHeader;
		pBootConfig->dByteCount = 16;
		pBootConfig->loadType = ROM_DMA_DONE_POLL_IRQDONE;
		pBootConfig->bootType = ADI_ROM_SECURE_BOOT_DIS;
		asm("NOP;");
	}

	return 0;
}

