/*****************************************************************************
 * ROM_CustomBoot.h
 *****************************************************************************/

#ifndef __ROM_CUSTOMBOOT_H__
#define __ROM_CUSTOMBOOT_H__

#include <cdef2156x_rom.h>
#include <def2156x_rom.h>
#include <drivers/ospi/adi_ospi.h>
#include <stdio.h>

#define START_ADDRESS 0x60000000

#include <drivers/ospi/adi_ospi.h>
#include <drivers\twi\adi_twi_2156x.h>
#include <services/dma/adi_dma.h>
#include <stdio.h>

uint8_t nDriver_Memory[ADI_OSPI_MEMORY_SIZE + ADI_SPU_MEMORY_SIZE];
//uint32_t HandleMem = 0;

/* Flash commands */
#define	FLASH_WR_EN					(0x06)  /*	Set Write Enable Latch */
#define FLASH_WR_DI					(0x04)  /*	Reset Write Enable Latch */
#define SPI_RDID            		(0x9F)  /*	Read Identification */
#define FLASH_RD_STAT1				(0x05)  /*	Read Status Register1 */
#define FLASH_RD_STAT2				(0x35)  /*	Read Status Register2 */
#define FLASH_WR_STAT				(0x01)	/*	Write status register */
#define FLASH_ENTER_QPI				(0x35)	/*	Enter Flash QPI mode */
#define FLASH_EXIT_QPI				(0xF5)	/*	Exit Flash QPI mode */

#define FLASH_RD_EVCR_MICRON		(0x65)	/*	Read Enhanced volatile configuration register Micron */
#define FLASH_WR_EVCR_MICRON		(0x61)	/*	Read Enhanced volatile configuration register Micron */

#define FLASH_WR_WRCR2				(0x72)	/*	MX66LM1G write WRCR2 register */
#define FLASH_RD_WRCR2				(0x71)	/*	MX66LM1G read WRCR2 register */

#define FLASH_RESETEN				(0x66)	/*	Flash Reset Enable */
#define FLASH_RESET					(0x99)	/*	Flash Reset */

#define	FLASH_READ					(0x03)	/*	Normal Read mode */
#define	FLASH_FST_READ				(0x0B)	/*	Fast Read mode */
#define	FLASH_SINGLE_DTRREAD		(0x0D)	/*	Fast Read mode */
#define	FLASH_DOR_READ      		(0x3B)	/*	Fast Read mode */
#define	FLASH_DOR_DTRREAD      		(0x3D)	/*	Fast Read mode */
#define	FLASH_DIOR_READ      		(0xBB)	/*	Fast Read mode */
#define	FLASH_DIOR_DTRREAD     		(0xBD)	/*	Fast Read mode */
#define	FLASH_QOR_READ      		(0x6B)	/*	Fast Read mode */
#define	FLASH_QOR_DTRREAD      		(0x6D)	/*	Fast Read mode */
#define	FLASH_QIOR_READ      		(0xEB)	/*	Fast Read mode */
#define	FLASH_QIOR_DTRREAD      	(0xED)	/*	Fast Read mode */
#define FLASH_OPI_READ_1			(0xEC)
#define FLASH_OPI_READ_2			(0x13)
#define FLASH_OPI_DTRREAD_1			(0xEE)
#define FLASH_OPI_DTRREAD_2			(0x11)

#define FLASH_BE					(0xD8)  /*  Flash Block Erase (64KB) command */
#define FLASH_PG_PRM				(0x02)	/*	Page Program */
#define FLASH_DI_PRM				(0xA2)	/*	Dual Input Page Program  (Data on 2 lines) */
#define FLASH_EDI_PRM				(0xD2)	/*	Extended Dual Input Page Program (Address also on 2 lines) */
#define FLASH_QI_PRM    			(0x32)	/*	Quad Input Page Program */
#define FLASH_EQI_PRM				(0x38)	/*	Extended Quad Input Page Program */
#define FLASH_EQI_PRM_MIC			(0x12)	/*	Extended Quad Input Page Program for Micron Flashes */


#define DUMMY_BYTE				    (0x00)	/* dummy word, used in read operations */
#define WIP                  	    (0x1)	/* Check the write in progress bit of the SPI status register */
#define WEL                  		(0x2)	/* Check the write enable bit of the SPI status register */


/* Octal STR/DTR mode selection enum for Flash device */
typedef enum
{
    /*! Flash is configured in STR OPI mode */
	STR_MODE,

    /*!  Flash is configured in DTR OPI mode */
	DTR_MODE,

} FLASH_OPI_MODE;


/* Function prototypes */
void ConfigSoftSwitches_MacronixFlash(void);
ADI_OSPI_RESULT Configure_Flash(ADI_OSPI_HANDLE const hDevice, ADI_OSPI_STIGCMD *pSTIG_Command);
ADI_OSPI_RESULT OPI_En_MX66LM1G(ADI_OSPI_HANDLE const hDevice, ADI_OSPI_STIGCMD *pSTIG_Command, FLASH_OPI_MODE Transfer_Mode);
void Select_FlashDevice(void);
int Ospi_initialize(void);


#endif /* __ROM_CUSTOMBOOT_H__ */
