#include <cdef2156x_rom.h>
#include <def2156x_rom.h>

typedef ROM_BOOT_RESULT (*_bootrom_jumptable_mdma_t)(ROM_DMA_MDMA_CONFIG * pDmaCfg);
#pragma inline
#pragma section("seg_ssl_code")
static ROM_BOOT_RESULT adi_rom_MemDma_new(ROM_DMA_MDMA_CONFIG * pDmaCfg){
   _bootrom_jumptable_mdma_t pMemDma =(_bootrom_jumptable_mdma_t)(FUNC_ROM_MDMA);
   return (*pMemDma)(pDmaCfg);
}

#pragma section("seg_ssl_code")
ROM_BOOT_RESULT bootInitFunction(ADI_ROM_BOOT_CONFIG * pBootConfig)
{


	ROM_BOOT_RESULT result = ROM_BOOT_SUCCESS;

	*pREG_SCB5_REMAP = 0x1;            /* Remap the AXI accesses to OSPI0 */
	/*Base address of the DMA channel associated with the boot peripheral*/
	pBootConfig->pDmaBaseRegister = ADI_DMA9_BASE;
	/*The page size for block based boot sources  */
	pBootConfig->dPageByteCount = 4;
	/*The data width of the boot source  */
	pBootConfig->uwDataWidth = 0; /* We are dealing with byte data */
	return result;

}

#pragma section("seg_ssl_code")
ROM_BOOT_RESULT bootConfigFunction(ADI_ROM_BOOT_CONFIG * pBootConfig)
{

	ROM_BOOT_RESULT result = ROM_BOOT_SUCCESS;

	/*Do pin mux and peripheral config. Also configuring the flash in octal mode needs to be done.*/
	ospiOctalConfig();
	return result;
}

#pragma section("seg_ssl_code")
void bootLoadFunction(ADI_ROM_BOOT_CONFIG * pBootConfig)
{

	  ROM_BOOT_RESULT result;

	  ROM_DMA_MDMA_CONFIG MdmaCfg;
	  MdmaCfg.pSource = pBootConfig->pSource;
	  MdmaCfg.pDestination = pBootConfig->pDestination;
	  MdmaCfg.ByteCount = pBootConfig->dByteCount;
	  MdmaCfg.eId = ROM_DMA_MDMA0;
	  MdmaCfg.eOperation = ROM_DMA_MEM_COPY;
	  MdmaCfg.eDoneDetect = pBootConfig->loadType;

	  result = adi_rom_MemDma_new(&MdmaCfg);

	  /* Update pSource pointer */
	  pBootConfig->pSource = (void *) ((uint32_t) pBootConfig->pSource + (pBootConfig->uwSrcModifyMult * pBootConfig->dByteCount));

}

#pragma section("seg_ssl_code")
void bootCleanUpFunction(ADI_ROM_BOOT_CONFIG * pBootConfig)
{
	asm("NOP;");
}

