#include <drivers/ospi/adi_ospi.h>
#include "stdio.h"

#define MDMA_CHANNEL_USED 	ADI_DMA_MEMDMA_S0

typedef enum
{
    /*! Flash is configured in STR OPI mode */
	STR_MODE,

    /*!  Flash is configured in DTR OPI mode */
	DTR_MODE,

} FLASH_OPI_MODE;

/* Flash commands */
#define	FLASH_WR_EN			(0x06)  //	Set Write Enable Latch
#define FLASH_WR_DI			(0x04)  //	Reset Write Enable Latch
#define SPI_RDID            (0x9F)  //	Read Identification
#define FLASH_RD_STAT1		(0x05)  //	Read Status Register1
#define FLASH_RD_STAT2		(0x35)  //	Read Status Register2
#define FLASH_WR_STAT		(0x01)	//	Write status register
#define FLASH_ENTER_QPI		(0x35)	//	Enter Flash QPI mode
#define FLASH_EXIT_QPI		(0xF5)	//	Exit Flash QPI mode

#define FLASH_RD_EVCR_MICRON		(0x65)	//	Read Enhanced volatile configuration register Micron
#define FLASH_WR_EVCR_MICRON		(0x61)	//	Read Enhanced volatile configuration register Micron

#define FLASH_WR_WRCR2		(0x72)	//	MX66LM1G write WRCR2 register
#define FLASH_RD_WRCR2		(0x71)	//	MX66LM1G read WRCR2 register

#define FLASH_RESETEN		(0x66)	//	Flash Reset Enable
#define FLASH_RESET			(0x99)	//	Flash Reset

#define FLASH_RDID  		(0x9F)  //	Flash Read Identification

#define	FLASH_READ					(0x03)	//	Normal Read mode
#define	FLASH_FST_READ				(0x0B)	//	Fast Read mode
#define	FLASH_SINGLE_DTRREAD		(0x0D)	//	Fast Read mode
#define	FLASH_DOR_READ      		(0x3B)	//	Fast Read mode
#define	FLASH_DOR_DTRREAD      		(0x3D)	//	Fast Read mode
#define	FLASH_DIOR_READ      		(0xBB)	//	Fast Read mode
#define	FLASH_DIOR_DTRREAD     		(0xBD)	//	Fast Read mode
#define	FLASH_QOR_READ      		(0x6B)	//	Fast Read mode
#define	FLASH_QOR_DTRREAD      		(0x6D)	//	Fast Read mode
#define	FLASH_QIOR_READ      		(0xEB)	//	Fast Read mode
#define	FLASH_QIOR_DTRREAD      	(0xED)	//	Fast Read mode
#define FLASH_OPI_READ_1			(0xEC)
#define FLASH_OPI_READ_2			(0x13)
#define FLASH_OPI_DTRREAD_1			(0xEE)
#define FLASH_OPI_DTRREAD_2			(0x11)

#define WEL                  (0x2)	//Check the write enable bit of the SPI status register


uint32_t HandleMem = 0;
uint8_t nDriver_Memory[ADI_OSPI_MEMORY_SIZE + ADI_SPU_MEMORY_SIZE];

uint8_t STIG_RW_Buffer[8];

ADI_OSPI_STIGCMD STIG_Command =  {		/*! command op-code */
										0x3,
										/*! 2nd command byte for commands with extended op-code,
											! Must be set to '0' for all normal commands */
										0,

										/*! Bus mode */
										ADI_OSPI_STIG_SINGLE,

										/*! Transfer mode */
										ADI_OSPI_TRANSFERMODE_STR,

										/*! Number of dummy bytes */
										0,

										/*! Whether Mode data will be sent */
										0,

										/*! Mode byte data */
										0,

										/*! Specifies whether the command has any data requirements */
										ADI_OSPI_STIG_NODATA,

										/*! Number of Data Bytes */
										0,

										/*! address will be sent */
										0,

										/*! Number of Address Bytes */
										ADI_OSPI_CMD_ADDR_SIZE_1,

										/*! flash command address */
										0,
										 /*! Pointer to the data buffer.
										  * Acts as write buffer for commands which send out data
										  * Acts as read buffer for commands which expect data in response
										  */
										STIG_RW_Buffer
									};

/*********************************************************************
 Function:       OPI_En_MX66LM1G

 Description:    Enables the OPI mode of MX66LM1G flash device
 *********************************************************************/
ADI_OSPI_RESULT OPI_En_MX66LM1G(ADI_OSPI_HANDLE const hDevice, ADI_OSPI_STIGCMD *pSTIG_Command, FLASH_OPI_MODE Transfer_Mode)
{
	ADI_OSPI_RESULT eResult = ADI_OSPI_SUCCESS;
	uint8_t Flash_Status[1];
	uint8_t Flash_WRCR2[1];

	/* Enable the Write to flash registers by sending FLASH_WR_EN(0x6) command */
	pSTIG_Command->CmdOpcode       = FLASH_WR_EN;
	pSTIG_Command->CmdOpcode2      = 0;
	pSTIG_Command->eBusMode        = ADI_OSPI_STIG_SINGLE;
	pSTIG_Command->eTransferMode   = ADI_OSPI_TRANSFERMODE_STR;
	pSTIG_Command->NumOfDummyCycles= 0;
	pSTIG_Command->ModeDataEnable  = 0;
	pSTIG_Command->ModeData		   = 0;
	pSTIG_Command->eDataMode	   = ADI_OSPI_STIG_NODATA;
	pSTIG_Command->DataSize		   = 1;
	pSTIG_Command->AddressEnable   = 0;
	pSTIG_Command->AddrSize		   = ADI_OSPI_CMD_ADDR_SIZE_1;
	pSTIG_Command->Address         = 0;
	pSTIG_Command->pBuffer         = 0;

	eResult = adi_ospi_StigCommand(hDevice, pSTIG_Command);
	if(eResult != ADI_OSPI_SUCCESS)
	{
		//printf("Failed to send the WREN command to flash. Error: %d \n", eResult);
		return eResult;
	}

	/* Initialize the STIG command to read Flash status */
	pSTIG_Command->CmdOpcode       = FLASH_RD_STAT1;
	pSTIG_Command->eDataMode	   = ADI_OSPI_STIG_READDATA_EN;
	pSTIG_Command->DataSize		   = 1;
	pSTIG_Command->pBuffer		   = Flash_Status;

	/* Initialize the STIG buffer */
	Flash_Status[0] = 0;

	/* Wait till WEL bit in flash status register is not set */
	while(!(Flash_Status[0] & WEL) )
	{
		eResult = adi_ospi_StigCommand(hDevice, pSTIG_Command);
		if(eResult != ADI_OSPI_SUCCESS)
		{
			//printf("Error in reading the Flash status register. Error: %d \n", eResult);
			return eResult;
		}
	}


	/* Initialize the STIG command to write to MX66LM1G WRCR2 register */
	pSTIG_Command->CmdOpcode       = FLASH_WR_WRCR2;
	pSTIG_Command->eDataMode	   = ADI_OSPI_STIG_WRITEDATA_EN;
	pSTIG_Command->DataSize		   = 1;
	pSTIG_Command->AddressEnable   = 1;
	pSTIG_Command->AddrSize		   = ADI_OSPI_CMD_ADDR_SIZE_4;
	pSTIG_Command->Address         = 0;
	pSTIG_Command->pBuffer		   = Flash_WRCR2;

	/* Update the value to be written to WRCR2 register */
	if(Transfer_Mode == STR_MODE)
	{
		Flash_WRCR2[0] =  0x1;
	}
	else
	{
		Flash_WRCR2[0] =  0x2;
	}

	eResult = adi_ospi_StigCommand(hDevice, pSTIG_Command);
	if(eResult != ADI_OSPI_SUCCESS)
	{
		//printf("Error in writing the Status register. Error: %d \n", eResult);
		return eResult;
	}


	/* Put some delay here */
	int Delay = 0;
	while(Delay++ < 10000){ asm("nop;"); }

	/* Read back the Config2 Register in STT_OPI mode */
	pSTIG_Command->CmdOpcode       = FLASH_RD_WRCR2;
	pSTIG_Command->CmdOpcode2      = 0x8E;
	pSTIG_Command->eBusMode        = ADI_OSPI_STIG_OPI;
	if(Transfer_Mode == STR_MODE)
	{
		pSTIG_Command->eTransferMode   = ADI_OSPI_TRANSFERMODE_STR;
	}
	else
	{
		pSTIG_Command->eTransferMode   = ADI_OSPI_TRANSFERMODE_DTR_P;
	}
	pSTIG_Command->NumOfDummyCycles= 20;
	pSTIG_Command->ModeDataEnable  = 0;
	pSTIG_Command->ModeData		   = 0;
	pSTIG_Command->eDataMode	   = ADI_OSPI_STIG_READDATA_EN;
	pSTIG_Command->DataSize		   = 1;
	pSTIG_Command->AddressEnable   = 1;
	pSTIG_Command->AddrSize		   = ADI_OSPI_CMD_ADDR_SIZE_4;
	pSTIG_Command->Address         = 0;
	pSTIG_Command->pBuffer         = Flash_WRCR2;

	Flash_WRCR2[0] = 0;

	eResult = adi_ospi_StigCommand(hDevice, pSTIG_Command);
	if(eResult != ADI_OSPI_SUCCESS)
	{
		return eResult;
	}

	if(Transfer_Mode == STR_MODE)
	{
		if(Flash_WRCR2[0] ==  0x1)
		{
			//printf("Flash configured in OSPI-STR mode correctly. Error: %d \n", eResult);

		}
	}
	else
	{
		if(Flash_WRCR2[0] ==  0x2)
		{
			//printf("Flash configured in OSPI-DTR mode correctly. Error: %d \n", eResult);
		}
	}


	return ADI_OSPI_SUCCESS;
}



/*********************************************************************
 Function:       Configure_Flash

 Description:    Configures the Flash device in a mode
  	  	  	  	 needed for the test
 *********************************************************************/
ADI_OSPI_RESULT Configure_Flash(ADI_OSPI_HANDLE const hDevice, ADI_OSPI_STIGCMD *pSTIG_Command)
{
	ADI_OSPI_RESULT eResult = ADI_OSPI_SUCCESS;


	/* Enable OPI mode of Macronix MX66LM1G45G Flash device */

	eResult = OPI_En_MX66LM1G(hDevice, pSTIG_Command, STR_MODE);

	return eResult;

}




/* Configures the Pin mux for OSPI */
void Init_pinmux(void)
{
	*pREG_PORTA_MUX = 0xAA555;
	*pREG_PORTA_FER = (BITM_PORT_FER_PX0 |BITM_PORT_FER_PX1| BITM_PORT_FER_PX2 | BITM_PORT_FER_PX3 | BITM_PORT_FER_PX4| BITM_PORT_FER_PX5 |
					   BITM_PORT_FER_PX6 | BITM_PORT_FER_PX7 | BITM_PORT_FER_PX8 | BITM_PORT_FER_PX9 );

}

//#pragma section("my_seg_int_code_sw")
void ospiOctalConfig()
{
	ADI_OSPI_RESULT eResult = ADI_OSPI_SUCCESS;
	uint8_t RDCR_value;
	ADI_OSPI_HANDLE *phDevice = &HandleMem;

	/* Initialize OSPI pin mux */
	Init_pinmux();           // Replace with GUI

	eResult = adi_ospi_Open(   (uint8_t *)nDriver_Memory + ADI_SPU_MEMORY_SIZE,
								ADI_OSPI_MEMORY_SIZE,
								OSPI_ADI_MDMANOTUSED,
								NULL,
								phDevice
							 );

	if(eResult != ADI_OSPI_SUCCESS)
	{

		return ;
	}

	eResult = adi_ospi_ConfigBaud(*phDevice, ADI_OSPI_BAUD_DIV_BY_8);
	if(eResult != ADI_OSPI_SUCCESS)
	{

		return ;
	}

	RDCR_value = 8;

	eResult = adi_ospi_UpdateReadDelay(*phDevice, RDCR_value);
	if(eResult != ADI_OSPI_SUCCESS)
	{

		return ;
	}

	/* Configures the flash device to enter the specific mode for the given test */
	Configure_Flash(*phDevice, &STIG_Command);


	*pREG_OSPI0_CTL		|=	BITM_OSPI_CTL_DACEN;

	*pREG_OSPI0_DRICTL	&=	~(BITM_OSPI_DRICTL_DMYRD|BITM_OSPI_DRICTL_DATATRNSFR|BITM_OSPI_DRICTL_ADDRTRNSFR|BITM_OSPI_DRICTL_OPCODERD);
	*pREG_OSPI0_DRICTL	|=	(0x14<<BITP_OSPI_DRICTL_DMYRD|3<<BITP_OSPI_DRICTL_DATATRNSFR|3<<BITP_OSPI_DRICTL_ADDRTRNSFR|0xEC<<BITP_OSPI_DRICTL_OPCODERD);

	*pREG_OSPI0_DSCTL &= ~BITM_OSPI_DSCTL_ADDRSZ;
	*pREG_OSPI0_DSCTL |= 3<<BITP_OSPI_DSCTL_ADDRSZ;

}

